"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "UpstashVectorStore", {
    enumerable: true,
    get: function() {
        return UpstashVectorStore;
    }
});
const _types = require("./types.js");
const _env = require("@llamaindex/env");
const _vector = require("@upstash/vector");
const _utils = require("./utils.js");
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interop_require_wildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {
        __proto__: null
    };
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
class UpstashVectorStore extends _types.BaseVectorStore {
    storesText = true;
    db;
    maxBatchSize;
    namespace;
    /**
   * @param namespace namespace to use
   * @param token upstash vector token. if not set, `process.env.UPSTASH_VECTOR_REST_TOKEN` is used.
   * @param endpoint upstash vector endpoint. If not set, `process.env.UPSTASH_VECTOR_REST_URL` is used.
   * @param maxBatchSize maximum number of vectors upserted at once. Default is 1000.
   *
   * @example
   * ```ts
   * const vectorStore = new UpstashVectorStore({ namespace: "my-namespace" })
   * ```
   */ constructor(params){
        super(params);
        this.namespace = params?.namespace ?? "";
        this.maxBatchSize = params?.maxBatchSize ?? 1000;
        const token = params?.token ?? (0, _env.getEnv)("UPSTASH_VECTOR_REST_TOKEN");
        const endpoint = params?.endpoint ?? (0, _env.getEnv)("UPSTASH_VECTOR_REST_URL");
        if (!token) {
            throw new Error("Must specify UPSTASH_VECTOR_REST_TOKEN via env variable.");
        }
        if (!endpoint) {
            throw new Error("Must specify UPSTASH_VECTOR_REST_URL via env variable.");
        }
        this.db = new _vector.Index({
            token,
            url: endpoint
        });
    }
    async getDb() {
        if (!this.db) {
            const { Index } = await Promise.resolve().then(()=>/*#__PURE__*/ _interop_require_wildcard(require("@upstash/vector")));
            this.db = new Index();
        }
        return this.db;
    }
    /**
   * Connects to the database specified in environment vars.
   * @returns A connection to the database, or the error encountered while connecting/setting up.
   */ client() {
        return this.getDb();
    }
    /**
   * Adds vector record(s) to the table.
   * @param embeddingResults The Nodes to be inserted, optionally including metadata tuples.
   * @returns ids of the embeddings (infered from the id_ field of embeddingResults objects)
   */ async add(embeddingResults) {
        if (embeddingResults.length == 0) {
            return [];
        }
        const nodes = embeddingResults.map(this.nodeToRecord);
        const result = await this.upsertInBatches(nodes);
        if (result != "OK") {
            throw new Error("Failed to save chunk");
        }
        return nodes.map((node)=>node.id).filter((id)=>!!id);
    }
    /**
   * Adds plain text record(s) to the table. Upstash take cares of embedding conversion.
   * @param text The Nodes to be inserted, optionally including metadata tuples.
   * @returns ids of the embeddings (infered from the id_ field of embeddingResults objects)
   */ async addPlainText(text) {
        if (text.length == 0) {
            return [];
        }
        const nodes = text.map(this.textNodeToRecord);
        const result = await this.upsertInBatches(nodes);
        if (result != "OK") {
            throw new Error("Failed to save chunk");
        }
        return nodes.map((node)=>node.id).filter((id)=>!!id);
    }
    async upsertInBatches(nodes) {
        const promises = [];
        for(let i = 0; i < nodes.length; i += this.maxBatchSize){
            const batch = nodes.slice(i, i + this.maxBatchSize);
            promises.push(this.db.upsert(batch, {
                namespace: this.namespace
            }));
        }
        const results = await Promise.all(promises);
        return results.every((result)=>result === "OK") ? "OK" : "NOT-OK";
    }
    /**
   * Deletes a single record from the database by id.
   * NOTE: Uses the collection property controlled by setCollection/getCollection.
   * @param refDocId Unique identifier for the record to delete.
   * @returns Promise that resolves if the delete query did not throw an error.
   */ async delete(refDocId) {
        await this.db.namespace(this.namespace).delete(refDocId);
    }
    /**
   * Deletes a single record from the database by id.
   * NOTE: Uses the collection property controlled by setCollection/getCollection.
   * @param refDocId Unique identifier for the record to delete.
   * @param deleteKwargs Required by VectorStore interface.  Currently ignored.
   * @returns Promise that resolves if the delete query did not throw an error.
   */ async deleteMany(refDocId) {
        await this.db.namespace(this.namespace).delete(refDocId);
    }
    /**
   * Query the vector store for the closest matching data to the query embeddings
   * @param query The VectorStoreQuery to be used
   * @param options Required by VectorStore interface.  Currently ignored.
   * @returns Zero or more Document instances with data from the vector store.
   */ async query(query, _options) {
        const filter = this.toUpstashFilter(query.filters);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const defaultOptions = {
            vector: query.queryEmbedding,
            topK: query.similarityTopK,
            includeVectors: true,
            includeMetadata: true,
            filter
        };
        const db = this.db;
        const results = await db.query(defaultOptions, {
            namespace: this.namespace
        });
        const nodes = results.map((result)=>{
            const node = (0, _utils.metadataDictToNode)(result.metadata ?? {}, {
                fallback: {
                    id: result.id,
                    metadata: result.metadata,
                    embedding: result.vector
                }
            });
            return node;
        });
        const ret = {
            nodes: nodes,
            similarities: results.map((row)=>row.score || 999),
            ids: results.map((row)=>String(row.id))
        };
        return ret;
    }
    toFilterString(filter) {
        return `${filter.key} ${filter.operator} ${filter.value}`;
    }
    toUpstashFilter(stdFilters) {
        if (!stdFilters?.filters) return;
        for (const item of stdFilters.filters){
            if (item.operator === "==") {
                //@ts-expect-error Upstash equal operator uses only one equal sign, so we have to replace it.
                item.operator = "=";
            }
        }
        const filterStrings = stdFilters.filters.map(this.toFilterString);
        if (filterStrings.length === 1) {
            return filterStrings[0];
        }
        return filterStrings.join(` ${stdFilters.condition ?? "and"} `);
    }
    nodeToRecord(node) {
        const id = node.id_.length ? node.id_ : null;
        return {
            // fixme: why id is possibly null?
            id: id,
            vector: node.getEmbedding(),
            metadata: (0, _utils.nodeToMetadata)(node)
        };
    }
    textNodeToRecord(node) {
        const id = node.id_.length ? node.id_ : null;
        return {
            // fixme: why id is possibly null?
            id: id,
            data: node.text,
            metadata: (0, _utils.nodeToMetadata)(node)
        };
    }
}
