import { VertexAI } from "@google-cloud/vertexai";
import { streamConverter } from "@llamaindex/core/utils";
import { getEnv, randomUUID } from "@llamaindex/env";
import { DEFAULT_SAFETY_SETTINGS, getFunctionCalls, getText } from "./utils.js";
/* To use Google's Vertex AI backend, it doesn't use api key authentication.
 *
 * To authenticate for local development:
 *
 *     ```
 *     npm install @google-cloud/vertexai
 *     gcloud auth application-default login
 *     ```
 * For production the prefered method is via a service account, more
 * details: https://cloud.google.com/docs/authentication/
 *
 * */ export class GeminiVertexSession {
    vertex;
    preview = false;
    constructor(options){
        const project = options?.project ?? getEnv("GOOGLE_VERTEX_PROJECT");
        const location = options?.location ?? getEnv("GOOGLE_VERTEX_LOCATION");
        if (!project || !location) {
            throw new Error("Set Google Vertex project and location in GOOGLE_VERTEX_PROJECT and GOOGLE_VERTEX_LOCATION env variables");
        }
        this.vertex = new VertexAI({
            ...options,
            project,
            location
        });
        this.preview = options?.preview ?? false;
    }
    getGenerativeModel(metadata) {
        if (this.preview) {
            return this.vertex.preview.getGenerativeModel({
                safetySettings: DEFAULT_SAFETY_SETTINGS,
                ...metadata
            });
        }
        return this.vertex.getGenerativeModel({
            safetySettings: DEFAULT_SAFETY_SETTINGS,
            ...metadata
        });
    }
    getResponseText(response) {
        return getText(response);
    }
    getToolsFromResponse(response) {
        return getFunctionCalls(response)?.map((call)=>({
                name: call.name,
                input: call.args,
                id: randomUUID()
            }));
    }
    async *getChatStream(result) {
        yield* streamConverter(result.stream, (response)=>{
            const tools = this.getToolsFromResponse(response);
            const options = tools?.length ? {
                toolCall: tools
            } : {};
            return {
                delta: this.getResponseText(response),
                raw: response,
                options
            };
        });
    }
    getCompletionStream(result) {
        return streamConverter(result.stream, (response)=>({
                text: this.getResponseText(response),
                raw: response
            }));
    }
}
